@@ -14,6 +14,7 @@ use tracing_subscriber::fmt::format::DefaultFields;
 use tracing_subscriber::fmt::MakeWriter;
 use tracing_subscriber::layer::SubscriberExt;
 
+use crate::filter::hide_indicatif_span_fields;
 use crate::span_ext::IndicatifSpanExt;
 use crate::suspend_tracing_indicatif;
 use crate::IndicatifLayer;
@@ -858,6 +859,50 @@ hello world
     );
 }
 
+#[test]
+fn test_with_span_field_formatter() {
+    let indicatif_layer = IndicatifLayer::new()
+        .with_span_field_formatter(hide_indicatif_span_fields(DefaultFields::new()));
+
+    let subscriber = tracing_subscriber::registry()
+        .with(tracing_subscriber::fmt::layer().with_writer(indicatif_layer.get_stderr_writer()))
+        .with(indicatif_layer);
+
+    tracing::subscriber::with_default(subscriber, || {
+        let _span = info_span!("foo");
+        _span.pb_start();
+
+        suspend_tracing_indicatif(|| {});
+    });
+}
+
+#[test]
+fn test_parent_span_enter_ordering() {
+    let (subscriber, _) = make_helpers(HelpersConfig::default());
+
+    tracing::subscriber::with_default(subscriber, || {
+        let grandparent_span = info_span!("grandparent");
+        let parent_span = info_span!(parent: &grandparent_span, "parent");
+        let child_span = info_span!(parent: &parent_span, "child");
+
+        let span1 = info_span!("span1");
+        span1.pb_start();
+        let span2 = info_span!("span2");
+        span2.pb_start();
+        let span3 = info_span!("span3");
+        span3.pb_start();
+        let span4 = info_span!("span4");
+        span4.pb_start();
+        let span5 = info_span!("span5");
+        span5.pb_start();
+
+        child_span.pb_start();
+        grandparent_span.pb_start();
+
+        drop(span1);
+    });
+}
+
 // These don't actually run anything, but exist to type check macros.
 #[allow(dead_code)]
 fn type_check_indicatif_println() {
